function [A,B,C,err_als_vec,temps]   =  admm_als_full(Y,iter_out,iter_in,rho,...
                                                               A_0,B_0,C_0)
%-------------------------------------------------------------------------%
% [A_c,B_c,C_c,err_als_vec]= admm_als(G,U,V,W,iter_out,iter_in,A_0,B_0,C_0)
%
% ADMM-based compressed and projected ALS algorithm for large-scale 3-way 
% nonnegative tensor decomposition.
%
% The inputs are:
% 
% - Y           : Data tensor;
% - iter_out    : maximum number of outer iterates (3 factors update);
% - iter_in     : number of maximum ADMM iterates for each factor;
% - rho         : ADMM parameter;
% - A_0,B_0,C_0 : initial parameters for the factors.
%
% The outputs are:
% 
% - A           : first factor.
% - B           : second factor.
% - C           : third factor.
% - err_als_vec : vector of errors.
%
% List of updates                 -     28/08/2014  -     R. Cabral Farias
%                                       Creation of the file 
%                                       20/04/2016  -     M. A. Veganzones
%                                                         R. C. Farias
%                                       Update to take into account diffe-
%                                       rent numbers of inner iterations
%                                       for ADMM
%                                       11/08/2016  -     J.E.Cohen
%                                       Uncompressed version
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Number of iterations for each evaluation of the reconstruction error
iter_err  =     1;
% Size of the filter for the relative error decrease
n_diff_err=     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_err  =     eps;
% Maximum number of iterations (/iter_err)
iter_out  =     ceil(iter_out/iter_err);
%-------------------------------------------------------------------------%

%--------------------------Tensor compression-----------------------------%
% Size of the tensor
dim_u     =     size(Y);
% Index for n-mode matricization
J_u       =     prod(dim_u)./dim_u;
% Data tensor norm
norm_Y    =     sqrt(Y(:)'*Y(:));
% Normalization
Y         =     Y/norm_Y;
% Number of components
R         =     size(A_0,2);
% Inverse of rho
rho_inv   =     1/rho;

% Three different unfoldings of the tensor
Y_mode    =     cell(3);
% Data tensor unfolding
Y_mode{1} =     reshape(permute(Y,[1,2,3]),dim_u(1),J_u(1)); 
Y1vec     =     Y_mode{1}(:);
Y_mode{2} =     reshape(permute(Y,[2,1,3]),dim_u(2),J_u(2));
Y_mode{3} =     reshape(permute(Y,[3,1,2]),dim_u(3),J_u(3));
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Filter for the relative error decrease
diff_err_filt   =     1000*ones(1,n_diff_err);
% Storing initial condition on the compressed space
A         =     A_0;
B         =     B_0;
C         =     C_0;
% Initial squared norm of the error
% Vector of squared norms of the error
err_als_vec     =     zeros(iter_out);
model           =     A_0*kr(C_0,B_0)';
err_als_vec(1)  =     norm_Y^2-2*model(:)'*norm_Y*Y1vec + sum(sum(model.^2));
err_als_old     =     err_als_vec(1);                   
%-------------------------------------------------------------------------%

fprintf('\n ADMM ALS\n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
while     (mean(diff_err_filt)>stop_diff &&  t<iter_out ...
          &&  err_als_old>stop_err   )
    % Outer iteration counter
    t     =     t+1;
    temps(1)  =     1;

    timer_ad=tic;   
    % ADMM (iter_err iterated)
    for   i           =     1:iter_err
    
        
    % Initialization of the projected variables
    A_u       =     zeros(dim_u(1),R);
    B_u       =     zeros(dim_u(2),R);
    C_u       =     zeros(dim_u(3),R);
    % Initialization of the dual variables
    mu_A      =     zeros(dim_u(1),R);
    mu_B      =     zeros(dim_u(2),R);
    mu_C      =     zeros(dim_u(3),R);
    
    % ADMM iterations for each factor

    % Factor A
    tt = 0;
    Y_CB  =     Y_mode{1}*kr(C,B);
    temp_CB     =     (C'*C).*(B'*B);
    while     (tt<iter_in)
    % Inner iteration counter
    tt    =     tt+1;
    
    % A_c factor update
    A     =     (Y_CB-0.5*(mu_A-rho*A_u))/...
                (((rho/2)*eye(R,R))+temp_CB);
    % A_u factor update
    A_u         =     A+rho_inv*mu_A;
    A_u(A_u<0)  =     0;
    % E_A update
    mu_A        =     mu_A+rho*(A-A_u);
    end
    % Normalization
    A     =     A.*repmat(1./sqrt(sum(A.^2)),dim_u(1),1);
    
    % Factor B
    tt = 0;
    Y_CA  =     Y_mode{2}*kr(C,A);
    temp_CA     =     (C'*C).*(A'*A);
    while     (tt<iter_in)
    % Inner iteration counter
    tt    =     tt+1;
          
    % B factor update
    B     =     (Y_CA-0.5*(mu_B-rho*B_u))/...
                (((rho/2)*eye(R,R))+temp_CA);
    % B_u factor update
    B_u         =     B+rho_inv*mu_B;
    B_u(B_u<0)  =     0;
    % E_B update
    mu_B        =     mu_B+rho*(B-B_u);
    end
    % Normalization
    B     =     B.*repmat(1./sqrt(sum(B.^2)),dim_u(2),1);  
    
    % Factor C
    tt = 0;
    Y_BA  =     Y_mode{3}*kr(B,A);
    temp_BA    =     (B'*B).*(A'*A);
    while     (tt<iter_in)     
    % Inner iteration counter     
    tt    =     tt+1;
          
    % C_c factor update
    C     =     (Y_BA-0.5*(mu_C-rho*C_u))/...
                (((rho/2)*eye(R,R))+temp_BA);
    % C_u factor update
    C_u         =     C+rho_inv*mu_C;
    C_u(C_u<0)  =     0;
    % E_C update
    mu_C   =     mu_C+rho*(C-C_u);
    end
    end
   
    temps(t+1)  =    toc(timer_ad); 
    
    % Squared norm of the error
    model           =     A_u*kr(C_u,B_u)';
    err_als         =     norm_Y^2*(1-2*model(:)'*Y1vec + sum(sum(model.^2)));
    
    % Evaluation of the relative error decrease
    diff_err_filt(1:end-1)  =     diff_err_filt(2:end);
    diff_err_filt(end)      =     abs((err_als-err_als_old)/err_als);
    
    % Storing the last error
    err_als_old       =     err_als;
    err_als_vec(t+1)  =     err_als;
    if mod(t*iter_err,100)  ==    0    
    % Printing the iterate error and step size
    fprintf('Reconstruction error : %g  \tit : %d\n',err_als,t*iter_err)
    end
    
end
% Projection
A(A<0)    =     0;
B(B<0)    =     0;
C(C<0)    =     0;
% Normalization of the components
C               =     C.*repmat(sqrt(sum(A.^2)).*sqrt(sum(B.^2)),dim_u(3),1)*norm_Y;
A               =     A.*repmat(1./sqrt(sum(A.^2)),dim_u(1),1);
B               =     B.*repmat(1./sqrt(sum(B.^2)),dim_u(2),1);
A(isnan(A))     =     0;
B(isnan(B))     =     0;
C(isnan(C))     =     0;
% Deleting zero error trailing at the end
err_als_vec(err_als_vec==0)=[];
%err_als_vec=[0 iter_err:iter_err:iter_err*t;err_als_vec];
%-------------------------------------------------------------------------%    