
%----------------------------admm_als_test.m------------------------------%
% In this file, ADMM ALS is tested and compared with PROCO-ALS and uncom-
% pressed ANLS. 
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% CP number of components (comment for large tensor test)
R         =     5;
% Uncompressed dimensions (comment for large tensor test)
dim_u     =     [200 200 200];
% Compressed dimensions (default is given by the rank) 
%                       (comment for large tensor test)
dim_c     =     [R R R];
% Number of block descent updates
iter_out  =     1000;
% Number of maximum inner iterates for ADMM
iter_in   =     4;
% ADMM parameter
rho       =     0.01;
% Gaussian noise standard deviation
sigma     =     0.00001;
%-------------------------------------------------------------------------%

%---------------------------Original CP model-----------------------------%
% CP model and noisy measurements generation (additive Gaussian noise)
% Unnormalized nonnegative factors
A   =     abs(randn(dim_u(1),R));
B   =     abs(randn(dim_u(2),R));
C   =     abs(randn(dim_u(3),R));
% Normalized nonnegative factors
A         =     A./ repmat(sum(A), dim_u(1), 1);
B         =     B./ repmat(sum(B), dim_u(2), 1);
% Tensor measurement
Y   =     reshape(A*transpose(kr(C,B)),dim_u)+sigma*randn(dim_u);
% Tensor compression
t_hosvd_start   =     tic;
[G,U,V,W] =     hosvd(Y,dim_c,'random');
t_hosvd         =     toc(t_hosvd_start);

% % Loading large tensor model - compressed core tensor G, compression matri-
% % ces U, V and W and uncompressed factors A, B and C.
% load      comp_tensor;
% % Uncompressed dimensions
% dim_u     =     [size(U,1) size(V,1) size(W,1)];
% % Compressed dimensions
% dim_c     =     [size(U,2) size(V,2) size(W,2)];
% % Rank
% R         =     size(A,2);
%-------------------------------------------------------------------------%

%-----------------------------Initialization------------------------------%
A_0 =     abs(randn(dim_u(1),R));
B_0 =     abs(randn(dim_u(2),R));
C_0 =     abs(randn(dim_u(3),R)).*repmat((sum(A_0).*sum(B_0)),dim_u(3),1);
A_0 =     A_0.*repmat(1./sum(A_0),dim_u(1),1);
B_0 =     B_0.*repmat(1./sum(B_0),dim_u(2),1);
%-------------------------------------------------------------------------%

%--------------------------------ADMM-ALS---------------------------------%
% Start time counter
t_admm_start                =     tic;
% Projected ALS
[A_admm,B_admm,C_admm,cost_admm]  =   admm_als(G,U,V,W,iter_out,iter_in,...
                                                          rho,A_0,B_0,C_0);
% Comment t_hosvd for large tensor test
% Stop time counter
t_admm          =     toc(t_admm_start);%+t_hosvd;
% Resulting factors
% Correcting permutation and sign ambiguities
[A_admm,B_admm,C_admm]=     amb_correct(A_admm,B_admm,C_admm,A,B,C);
% Total squared error on the factors
tse_admm        =           sum((A(:)-A_admm(:)).^2)+...
                            sum((B(:)-B_admm(:)).^2)+...
                            sum((C(:)-C_admm(:)).^2);
%-------------------------------------------------------------------------%

%---------------------------------PROCO-ALS-------------------------------%
% Start time counter
t_proco_start               =     tic;
% Projected ALS
[A_proco,B_proco,C_proco,cost_proco,~,~,~,~]=proco_als(0,iter_out,dim_c,...
                                                     'standard','given',...
                                                            A_0,B_0,C_0,...
                                                                0,G,U,V,W);
% Comment t_hosvd for large tensor test
% Stop time counter
t_proco         =     toc(t_proco_start);%+t_hosvd;
% Resulting factors
% Correcting permutation and sign ambiguities
[A_proco,B_proco,C_proco]   =   amb_correct(A_proco,B_proco,C_proco,A,B,C);
% Total squared error on the factors
tse_proco       =           sum((A(:)-A_proco(:)).^2)+...
                            sum((B(:)-B_proco(:)).^2)+...
                            sum((C(:)-C_proco(:)).^2);
%-------------------------------------------------------------------------%

% Comment this part for the test with the large tensor
% %-----------------------------------ANLS----------------------------------%
% % Start time counter
% t_anls_start                =     tic;
% % Projected ALS
% [A_anls,B_anls,C_anls,cost_anls]  =     anls(Y,iter_out,A_0,B_0,C_0);
% % Stop time counter
% t_anls          =     toc(t_proco_start);
% % Resulting factors
% % Correcting permutation and sign ambiguities
% [A_anls,B_anls,C_anls]=     amb_correct(A_anls,B_anls,C_anls,A,B,C);
% % Total squared error on the factors
% tse_anls        =           sum((A(:)-A_anls(:)).^2)+...
%                             sum((B(:)-B_anls(:)).^2)+...
%                             sum((C(:)-C_anls(:)).^2);
% %-------------------------------------------------------------------------%


%--------------------------------Results----------------------------------%
% % Comment these lines for the test with the large tensor
% fprintf('\n \t \t \t ADMM ALS \t \t PROCO-ALS \t \t ANLS \t \t\n')
% fprintf('Exec. time \t \t  %g \t \t %g \t \t %g\n',t_admm,t_proco,t_anls)
% fprintf('Total squared error \t %g \t \t %g \t \t %g\n',tse_admm,...
%                                                        tse_proco,tse_anls)
% Uncomment the lines below for the test with large tensor
fprintf('\n \t \t \t ADMM ALS \t \t PROCO-ALS \t \t\n')
fprintf('Exec. time \t \t  %g \t \t %g\n',t_admm,t_proco)
fprintf('Total squared error \t %g \t \t %g\n',tse_admm,tse_proco)                                                 
%-------------------------------------------------------------------------%