function [A,B,C,err_als,temps]     =     dk_als(G,iter,inner,dim_c,dim_u,A_0,B_0,C_0,U,V,W,T)
%-------------------------------------------------------------------------%
% [A_c,B_c,C_c,err_als_vec]=     dk(Y,iter,dim_c,A_0,B_0,C_0)
%
% Compressed and projected ALS algorithm for large-scale 3-way nonnegative
% tensor decomposition.
%
% The inputs are:
% 
% - Y           : data block.
% - iter        : maximum number of iterates.
% - dim_c       : vector with the dimensions of the compressed model.
% - A_0,B_0,C_0 : initial parameters for the factors.
%
% The outputs are:
% 
% - A           : first factor.
% - B           : second factor.
% - C           : third factor.
% - err_als     : final error.
% - temps       : time per iteration
%
% List of updates                 -     28/08/2014  -     Rodrigo Cabral
%                                       Creation of the file
%                                 -     02/10/2014  -     Jeremy Cohen
%                                       Reshaping, modified outputs
%                                 -     07/01/2015  -     Jeremy Cohen
%                                       Added Lambda as output
%                                 -     08/06/2015  -     Jeremy Cohen
%                                       QuadProg version
%                                 -     06/07/2015  -     Jeremy Cohen
%                                       Added Time output.
%                                 -     06/07/2015  -     Jeremy Cohen
%                                       Dykstra version.
%-------------------------------------------------------------------------%


%--------------------------Tensor compression-----------------------------%
R    =    size(A_0,2);
% Index for n-mode matricization of compressed data
J_c  =    prod(dim_c)./dim_c;
% Truncated core tensors (compressed data)
norm_G          =     sqrt(G(:)'*G(:));
% Normalized G
G               =     G./norm_G;

% Three different unfoldings of the tensor
G_mode    =     cell(3);
% Data tensor unfolding
G_mode{1} =     reshape(permute(G,[1,2,3]),dim_c(1),J_c(1));  
G_mode{2} =     reshape(permute(G,[2,1,3]),dim_c(2),J_c(2));
G_mode{3} =     reshape(permute(G,[3,1,2]),dim_c(3),J_c(3));



%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%

% Storing initial condition on the compressed space
A_c  =    (U')*A_0;
B_c  =    (V')*B_0;
C_c  =    (W')*C_0;

%-------------------------------------------------------------------------%

fprintf('\n\n')
fprintf('\n Dykstra QuadProg  - DKQP \n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
temps(1)  =     1;
err_als(1) =     sum((T(:)-kr(C_0,B_0,A_0)*ones(R,1)).^2);

while     t<iter
    
time = tic;    
    
    % Iteration counter
    t=t+1;
      
    % A factor update
    temp_CB     =  (C_c'*C_c).*(B_c'*B_c);
    A_c         =  (G_mode{1}*kr(C_c,B_c))/temp_CB;
    % Projection to have nonnegative factors on the uncompressed space
    resi = zeros(dim_c(1),R,dim_u(1));
    for i1=1:inner
        for i2=1:dim_u(1)
        %correction by previous residual
        A_c_new = A_c - resi(:,:,i2);
        %projection on halfspace
        temp_A = U(i2,:)*A_c_new;
            if temp_A < 0
            A_c_new = A_c_new - U(i2,:)'*temp_A/norm(U(i2,:));%project
            end
        %new residual
        resi(:,:,i2) = A_c_new - A_c;
        A_c = A_c_new;
        end
    end
    % Normalisation
    A_c       =     A_c.*repmat(1./sqrt(sum(A_c.^2)),dim_c(1),1);

    % B factor update
    temp_CA     =  (C_c'*C_c).*(A_c'*A_c);
    B_c         =  (G_mode{2}*kr(C_c,A_c))/temp_CA;
    % Projection to have nonnegative factors on the uncompressed space
    resi = zeros(dim_c(2),R,dim_u(2));
    for i1=1:inner
        for i2=1:dim_u(2)
        %correction by previous residual
        B_c_new = B_c - resi(:,:,i2);
        %projection on halfspace
        temp_B = V(i2,:)*B_c_new;
            if temp_B < 0
            B_c_new = B_c_new - V(i2,:)'*temp_B/norm(V(i2,:));%project
            end
        %new residual
        resi(:,:,i2) = B_c_new - B_c;
        B_c = B_c_new;
        end
    end
    % Normalisation
    B_c       =     B_c.*repmat(1./sqrt(sum(B_c.^2)),dim_c(2),1);       
    
    % C factor update
    temp_BA     =     (B_c'*B_c).*(A_c'*A_c);
    G_BA        =     G_mode{3}*kr(B_c,A_c);
    C_c         =     G_BA/temp_BA;
    % Projection to have nonnegative factors on the uncompressed space
    resi = zeros(dim_c(3),R,dim_u(3));
    for i1=1:inner
        for i2=1:dim_u(3)
        %correction by previous residual
        C_c_new = C_c - resi(:,:,i2);
        %projection on halfspace
        temp_C = W(i2,:)*C_c_new;
            if temp_C < 0
            C_c_new = C_c_new - W(i2,:)'*temp_C/norm(W(i2,:));%project
            end
        %new residual
        resi(:,:,i2) = C_c_new - C_c;
        C_c = C_c_new;
        end
    end
    
temps(t+1)=toc(time); 
    
err_als(t+1) =     sum((T(:)-norm_G*kr(W*C_c,V*B_c,U*A_c)*ones(R,1)).^2);    
%err_als(t+1) =     sum(sum((G_mode{1}-A_c*kr(C_c,B_c)').^2)); 
 
end

A   =     U*A_c;
B   =     V*B_c;
C   =     W*C_c;
A(A<0)    =     0;
B(B<0)    =     0;
C(C<0)    =     0;

norm_A_u  =     sqrt(sum(A.^2));
norm_B_u  =     sqrt(sum(B.^2));

A   =     A.*repmat(1./norm_A_u,dim_u(1),1);
B   =     B.*repmat(1./norm_B_u,dim_u(2),1);
C   =     C.*repmat(norm_G*norm_A_u.*norm_B_u,dim_u(3),1);


fprintf('Reconstruction error : %g  ',err_als(end)) 

A(isnan(A))     =     0;
B(isnan(B))     =     0;
C(isnan(C))     =     0;
%-------------------------------------------------------------------------%    
end