function [A,B,C,err_cp_vec,temps]  = prox_cp(Y,type,iter_out,iter_in...
                                             ,U,V,W,prox_A,prox_B,prox_C...
                                                          ,rho,tau,sigma...
                                                      ,A_p_0,B_p_0,C_p_0...
                                                        ,A_d_0,B_d_0,C_d_0,dim_u,T)
%-------------------------------------------------------------------------%
% [A_cp,B_cp,C_cp,err_cp]  =             prox_cp(Y,type,iter_out,iter_in...
%                                            ,U,V,W,prox_A,prox_B,prox_C...
%                                                         ,rho,tau,sigma...
%                                                     ,A_p_0,B_p_0,C_p_0...
%                                                       ,A_d_0,B_d_0,C_d_0)
%
% Primal-dual proximal algorithms for constrained third order CP decomposi-
% tion.
%
% The inputs are:
% 
% - Y           : data block;
% - type        : type of primal-dual proximal algorithm;
%
%                 For type='gradient', the data fitting term is considered
%                 as a smooth function and alternating gradient gradient
%                 is used;
%
%                 For type='proximal', the data fitting term is considered
%                 as a smooth function and alternating gradient gradient
%                 is used;
%
%                 For type='all-at-once', the data fitting term is conside-
%                 red as a smooth function and an all-at-once gradient is
%                 used;
% - iter_out    : number of outer iterations (update of all factors);
% - iter_in     : number of inner iteration  (required for 
%                 type='gradient' and 'proximal');
% - U,V,W       : Cells with operators for the three factors A, B, C;
% - prox_A,...  : Cells with handles for the proximal mappings for the
%                 constraints. The cell lengths must match those of U,V,
%                 and W;
% - rho,tau,... : tuning parameters of the primal-dual proximal algorithms;
% - A_p_0,...   : initial primal factors;
% - A_d_0,...   : initial dual factors.
% 
%
% The outputs are:
% 
% - A_cp,...    : CP factors;
% - err_cp_vec  : vector with objective values.
%
% List of updates                 -     06/07/2015  -     J. E. Cohen
%                                       Creation of the file
%                                 -     07/10/2015  -     R. C. Farias
%                                       Integration of the three proximal
%                                       algorithms in one function
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Number of iterations for each evaluation of the reconstruction error
iter_err  =     1;
% Size of the filter for the relative error decrease
n_diff_err=     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_err  =     eps;
% Truncated core tensors (compressed data)
norm_Y          =     sqrt(Y(:)'*Y(:));
% Normalized core
Y               =     Y./norm_Y;
R         =     size(A_p_0,2);
%-------------------------------------------------------------------------%

%---------------------------Initialization--------------------------------%
% Number of operators and proximal mappings
M_A       =     length(prox_A);
M_B       =     length(prox_B);
M_C       =     length(prox_C);
% Size of the tensor
dim       =     size(Y);
% Index for n-mode matricization
prod_dim  =     prod(dim)./dim;
% Maximum number of iterations (/iter_err)
iter_out  =     ceil(iter_out/iter_err);
% Three different unfoldings of the compressed tensor
Y_mode    =     cell(1,3);
% Data tensor unfolding
Y_mode{1} =     reshape(permute(Y,[1,2,3]),dim(1),prod_dim(1));  
Y_mode{2} =     reshape(permute(Y,[2,1,3]),dim(2),prod_dim(2));
Y_mode{3} =     reshape(permute(Y,[3,1,2]),dim(3),prod_dim(3));
% Initial primal factors
A_p =    (U{1}')*A_p_0; 
B_p =    (V{1}')*B_p_0;
C_p =    (W{1}')*C_p_0;
% Initial dual factors
A_d       =     cell(1,M_A);
for i     =     1:M_A
A_d{i}    =     A_d_0{i};
end
B_d       =     cell(1,M_B);
for i     =     1:M_B
B_d{i}    =     B_d_0{i};
end
C_d       =     cell(1,M_C);
for i     =     1:M_B
C_d{i}    =     C_d_0{i};
end
% Allocation
prox_map_A=     cell(1,M_A);
prox_map_B=     cell(1,M_B);
prox_map_C=     cell(1,M_C);
% Squared norm of the data
snorm_Y   =     Y_mode{1}(:)'*Y_mode{1}(:);
% Filter for the relative error decrease
diff_err_filt   =     1000*ones(1,n_diff_err);
% Initial squared norm of the error
res             =     T(:)-kr(C_p_0,B_p_0,A_p_0)*ones(R,1);
err_cp_old     =     (res')*res;
% Vector of squared norms of the error
err_cp_vec      =     zeros(1,iter_out+1);
% First error
err_cp_vec(1)   =     err_cp_old;
fprintf('\nProx-ALS: %s version\n',type)
fprintf('----\n')
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
switch(type)

    
%---------------------------Alternating Gradient--------------------------%
case('gradient')
%--------------------------Algorithm iterates-----------------------------%
t         =     0;
temps(1)  =     1;

while     (mean(diff_err_filt)>stop_diff && t<iter_out &&  ...
          err_cp_old>stop_err   )
    % Iteration counter (counts/iter_map)
    t=    t+1;
    tic
    % Coupled ALS (10 iterates)
    for   i           =     1:iter_err
          % Update of factor A
          % Khatri-Rao and auxiliary quantities
          kr_CB       =     kr(C_p,B_p);
          temp_CB     =     transpose(kr_CB)*kr_CB;
          Y_CB        =     Y_mode{1}*kr_CB;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in   
          % Effect of the dual variables on the primal
          U_t         =     cellfun(@transpose,U,'UniformOutput',0);
          UAd         =     cellfun(@(u,a) u*a,U_t,A_d,'UniformOutput',0);
          sumUAd      =     sum(cat(3,UAd{:}),3);
          % Partial update of the primal
          A_p_t       =     A_p+tau*(Y_CB-(A_p*temp_CB)-sumUAd);
          % Dual factors correction
          A_d_corr    =     cellfun(@(a,u) a+sigma*u*(2*A_p_t-A_p),A_d,...
                                                      U,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_A
          prox_map_A{k}     =     prox_A{k}(A_d_corr{k});
          end
          
          % Partial update of the dual
          A_d_t       =     cellfun(@(a,p) a-p,A_d_corr,prox_map_A,...
                                                        'UniformOutput',0);
          % Final updates
          A_p         =     rho*A_p_t+(1-rho)*A_p;
          A_d         =     cellfun(@(a_d_t,a_d) rho*a_d_t+(1-rho)*a_d,...
                                              A_d_t,A_d,'UniformOutput',0);
          end
          
          % Update of factor B
          % Khatri-Rao and auxiliary quantities
          kr_CA       =     kr(C_p,A_p);
          temp_CA     =     transpose(kr_CA)*kr_CA;
          Y_CA        =     Y_mode{2}*kr_CA;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in
          % Effect of the dual variables on the primal
          V_t         =     cellfun(@transpose,V,'UniformOutput',0);
          VBd         =     cellfun(@(v,b) v*b,V_t,B_d,'UniformOutput',0);
          sumVBd      =     sum(cat(3,VBd{:}),3);
          % Partial update of the primal
          B_p_t       =     B_p+tau*(Y_CA-(B_p*temp_CA)-sumVBd);    
          % Dual factors correction
          B_d_corr    =     cellfun(@(b,v) b+sigma*v*(2*B_p_t-B_p),B_d,...
                                                      V,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_B
          prox_map_B{k}     =     prox_B{k}(B_d_corr{k});
          end
          % Partial update of the dual
          B_d_t       =     cellfun(@(b,p) b-p,B_d_corr,prox_map_B,...
                                                        'UniformOutput',0);
          % Final updates
          B_p         =     rho*B_p_t+(1-rho)*B_p;
          B_d         =     cellfun(@(b_d_t,b_d) rho*b_d_t+(1-rho)*b_d,...
                                              B_d_t,B_d,'UniformOutput',0);
          end
          
          % Update of factor C
          % Khatri-Rao and auxiliary quantities
          kr_BA       =     kr(B_p,A_p);
          temp_BA     =     transpose(kr_BA)*kr_BA;
          Y_BA        =     Y_mode{3}*kr_BA;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in
          % Effect of the dual variables on the primal
          W_t         =     cellfun(@transpose,W,'UniformOutput',0);
          WCd         =     cellfun(@(w,c) w*c,W_t,C_d,'UniformOutput',0);
          sumWCd      =     sum(cat(3,WCd{:}),3);
          % Partial update of the primal
          C_p_t       =     C_p+tau*(Y_BA-(C_p*temp_BA)-sumWCd);    
          % Dual factors correction
          C_d_corr    =     cellfun(@(c,w) c+sigma*w*(2*C_p_t-C_p),C_d,...
                                                      W,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_C
          prox_map_C{k}     =     prox_C{k}(C_d_corr{k});
          end
          % Partial update of the dual
          C_d_t      =    cellfun(@(c,p) c-p,C_d_corr,prox_map_C,...
                                                        'UniformOutput',0);
          % Final updates
          C_p        =    rho*C_p_t+(1-rho)*C_p;
          C_d        =    cellfun(@(c_d_t,c_d) rho*c_d_t+(1-rho)*c_d,...
                                              C_d_t,C_d,'UniformOutput',0);
          end
    end
temps(t+1) = toc;     
    % Squared norm of the error (it uses some precalculated quantities)
    err_cp                  =     sum((T(:)-norm_Y*kr(W{1}*C_p,V{1}*B_p,U{1}*A_p)*ones(R,1)).^2);
    
    % Evaluation of the relative error decrease
    diff_err_filt(1:end-1)  =     diff_err_filt(2:end);
    diff_err_filt(end)      =     abs((err_cp-err_cp_old)/err_cp(1));
    
    % Storing the last error
    err_cp_old       =     err_cp;
    err_cp_vec(t+1)  =     err_cp;
    
    % Objective value evaluation
    if mod(t*iter_err,100)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('Objective value : %g  \tit : %d\n',err_cp,t*iter_err)
    end
  
    
end
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


%---------------------------------Proximal--------------------------------%
case('proximal')
%--------------------------Algorithm iterates-----------------------------%
Id        =     eye(size(A_p_0,2));
t         =     0;
temps(1)  =     1;

while     (mean(diff_err_filt)>stop_diff && t<iter_out &&  ...
          err_cp_old>stop_err   )
    % Iteration counter (counts/iter_map)
    t=    t+1;
    tic
    % Coupled ALS (10 iterates)
    for   i           =     1:iter_err
          % Update of factor A
          % Khatri-Rao and auxiliary quantities
          temp_CB     =     (C_p'*C_p).*(B_p'*B_p);
          Y_CB        =     Y_mode{1}*kr(C_p,B_p);
          A_div       =     Id+tau*temp_CB;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in   
          % Effect of the dual variables on the primal
          U_t         =     cellfun(@transpose,U,'UniformOutput',0);
          UAd         =     cellfun(@(u,a) u*a,U_t,A_d,'UniformOutput',0);
          sumUAd      =     sum(cat(3,UAd{:}),3);
          % Partial update of the primal
          A_p_t       =     (A_p+tau*(Y_CB-sumUAd))/A_div;
          % Dual factors correction
          A_d_corr    =     cellfun(@(a,u) a+sigma*u*(2*A_p_t-A_p),A_d,...
                                                      U,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_A
          prox_map_A{k}     =     prox_A{k}(A_d_corr{k});
          end
          
          % Partial update of the dual
          A_d_t       =     cellfun(@(a,p) a-p,A_d_corr,prox_map_A,...
                                                        'UniformOutput',0);
          % Final updates
          A_p         =     rho*A_p_t+(1-rho)*A_p;
          A_d         =     cellfun(@(a_d_t,a_d) rho*a_d_t+(1-rho)*a_d,...
                                              A_d_t,A_d,'UniformOutput',0);
          end
          
          % Update of factor B
          % Khatri-Rao and auxiliary quantities
          temp_CA     =     (C_p'*C_p).*(A_p'*A_p);
          Y_CA        =     Y_mode{2}*kr(C_p,A_p);
          B_div       =     Id+tau*temp_CA;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in
          % Effect of the dual variables on the primal
          V_t         =     cellfun(@transpose,V,'UniformOutput',0);
          VBd         =     cellfun(@(v,b) v*b,V_t,B_d,'UniformOutput',0);
          sumVBd      =     sum(cat(3,VBd{:}),3);
          % Partial update of the primal
          B_p_t       =     (B_p+tau*(Y_CA-sumVBd))/B_div;    
          % Dual factors correction
          B_d_corr    =     cellfun(@(b,v) b+sigma*v*(2*B_p_t-B_p),B_d,...
                                                      V,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_B
          prox_map_B{k}     =     prox_B{k}(B_d_corr{k});
          end
          % Partial update of the dual
          B_d_t       =     cellfun(@(b,p) b-p,B_d_corr,prox_map_B,...
                                                        'UniformOutput',0);
          % Final updates
          B_p         =     rho*B_p_t+(1-rho)*B_p;
          B_d         =     cellfun(@(b_d_t,b_d) rho*b_d_t+(1-rho)*b_d,...
                                              B_d_t,B_d,'UniformOutput',0);
          end
          
          % Update of factor C
          % Khatri-Rao and auxiliary quantities
          temp_BA     =     (B_p'*B_p).*(A_p'*A_p);
          Y_BA        =     Y_mode{3}*kr(B_p,A_p);
          C_div       =     Id+tau*temp_BA;
          % Iterates of the primal-dual proximal update
          for         j     =     1:iter_in
          % Effect of the dual variables on the primal
          W_t         =     cellfun(@transpose,W,'UniformOutput',0);
          WCd         =     cellfun(@(w,c) w*c,W_t,C_d,'UniformOutput',0);
          sumWCd      =     sum(cat(3,WCd{:}),3);
          % Partial update of the primal
          C_p_t       =     (C_p+tau*(Y_BA-sumWCd))/C_div;    
          % Dual factors correction
          C_d_corr    =     cellfun(@(c,w) c+sigma*w*(2*C_p_t-C_p),C_d,...
                                                      W,'UniformOutput',0);
          % Evaluation of the proximal mappings
          for   k     =     1:M_C
          prox_map_C{k}     =     prox_C{k}(C_d_corr{k});
          end
          % Partial update of the dual
          C_d_t      =    cellfun(@(c,p) c-p,C_d_corr,prox_map_C,...
                                                        'UniformOutput',0);
          % Final updates
          C_p        =    rho*C_p_t+(1-rho)*C_p;
          C_d        =    cellfun(@(c_d_t,c_d) rho*c_d_t+(1-rho)*c_d,...
                                              C_d_t,C_d,'UniformOutput',0);
          end
    end
temps(t+1) = toc;     
    % Squared norm of the error (it uses some precalculated quantities)
    %err_cp                  =     snorm_Y-2*sum(sum(Y_BA.*C_p))+...
       %                           sum(sum((C_p'*C_p).*temp_BA));
    err_cp                  =     sum((T(:)-norm_Y*kr(W{1}*C_p,V{1}*B_p,U{1}*A_p)*ones(R,1)).^2);
                                  
    % Evaluation of the relative error decrease
    diff_err_filt(1:end-1)  =     diff_err_filt(2:end);
    diff_err_filt(end)      =     abs((err_cp-err_cp_old)/err_cp(1));
    
    % Storing the last error
    err_cp_old       =     err_cp;
    err_cp_vec(t+1)  =     err_cp;
    
    % Objective value evaluation
    if mod(t*iter_err,100)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('Objective value : %g  \tit : %d\n',err_cp,t*iter_err)
    end
   
    
end
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


%--------------------------------All-at-once------------------------------%
case('all-at-once')
    t         =     0;
    temps(1)  =     1;
    % Auxiliary quantities
    temp_CB     =     (C_p'*C_p).*(B_p'*B_p);
          
    temp_CA     =     (C_p'*C_p).*(A_p'*A_p);
          
    temp_BA     =     (B_p'*B_p).*(A_p'*A_p);
          
    Y_CB        =     Y_mode{1}*kr(C_p,B_p);
          
    Y_CA        =     Y_mode{2}*kr(C_p,A_p);
          
    Y_BA        =     Y_mode{3}*kr(B_p,A_p);
while     (mean(diff_err_filt)>stop_diff && t<iter_out &&  ...
          err_cp_old>stop_err   )
    % Iteration counter (counts/iter_map)
    t=    t+1;
    tic
    % Coupled ALS (10 iterates)
    for   i           =     1:iter_err
        
          
          
          % Gradients
          gradA       =     -Y_CB+A_p*temp_CB;
          
          gradB       =     -Y_CA+B_p*temp_CA;
          
          gradC       =     -Y_BA+C_p*temp_BA;
          
          % Effect of the dual variables on the primal
          U_t         =     cellfun(@transpose,U,'UniformOutput',0);
          UAd         =     cellfun(@(u,a) u*a,U_t,A_d,'UniformOutput',0);
          sumUAd      =     sum(cat(3,UAd{:}),3);
          
          V_t         =     cellfun(@transpose,V,'UniformOutput',0);
          VBd         =     cellfun(@(v,b) v*b,V_t,B_d,'UniformOutput',0);
          sumVBd      =     sum(cat(3,VBd{:}),3);
          
          W_t         =     cellfun(@transpose,W,'UniformOutput',0);
          WCd         =     cellfun(@(w,c) w*c,W_t,C_d,'UniformOutput',0);
          sumWCd      =     sum(cat(3,WCd{:}),3);
          
          % All-at-once primal step
          A_p_t       =     A_p-tau*(gradA+sumUAd);
          
          B_p_t       =     B_p-tau*(gradB+sumVBd);
          
          C_p_t       =     C_p-tau*(gradC+sumWCd);
          
          % Dual factors correction
          A_d_corr    =     cellfun(@(a,u) a+sigma*u*(2*A_p_t-A_p),A_d,...
                                                      U,'UniformOutput',0);
                                                  
          B_d_corr    =     cellfun(@(b,v) b+sigma*v*(2*B_p_t-B_p),B_d,...
                                                      V,'UniformOutput',0);
                                                  
          C_d_corr    =     cellfun(@(c,w) c+sigma*w*(2*C_p_t-C_p),C_d,...
                                                      W,'UniformOutput',0);                                        
                                                  
          % Evaluation of the proximal mappings
          for   k     =     1:M_A
          prox_map_A{k}     =     prox_A{k}(A_d_corr{k});
          end
          
          for   k     =     1:M_B
          prox_map_B{k}     =     prox_B{k}(B_d_corr{k});
          end
          
          for   k     =     1:M_C
          prox_map_C{k}     =     prox_C{k}(C_d_corr{k});
          end
          
          % Partial update of the dual
          A_d_t       =     cellfun(@(a,p) a-p,A_d_corr,prox_map_A,...
                                                        'UniformOutput',0);
                                                    
          B_d_t       =     cellfun(@(b,p) b-p,B_d_corr,prox_map_B,...
                                                        'UniformOutput',0);
                                                    
          C_d_t      =    cellfun(@(c,p) c-p,C_d_corr,prox_map_C,...
                                                        'UniformOutput',0);
                                                    
          % Final updates
          A_p         =     rho*A_p_t+(1-rho)*A_p;
          A_d         =     cellfun(@(a_d_t,a_d) rho*a_d_t+(1-rho)*a_d,...
                                              A_d_t,A_d,'UniformOutput',0);
                                          
          B_p         =     rho*B_p_t+(1-rho)*B_p;
          B_d         =     cellfun(@(b_d_t,b_d) rho*b_d_t+(1-rho)*b_d,...
                                              B_d_t,B_d,'UniformOutput',0);
                                          
          C_p        =    rho*C_p_t+(1-rho)*C_p;
          C_d        =    cellfun(@(c_d_t,c_d) rho*c_d_t+(1-rho)*c_d,...
                                              C_d_t,C_d,'UniformOutput',0);
                                          
          % Auxiliary quantities for the next iterate and cost evaluation
          temp_CB     =     (C_p'*C_p).*(B_p'*B_p);
          
          temp_CA     =     (C_p'*C_p).*(A_p'*A_p);
          
          temp_BA     =     (B_p'*B_p).*(A_p'*A_p);
          
          Y_CB        =     Y_mode{1}*kr(C_p,B_p);
          
          Y_CA        =     Y_mode{2}*kr(C_p,A_p);
          
          Y_BA        =     Y_mode{3}*kr(B_p,A_p);
    end
temps(t+1) = toc; 
    % Squared norm of the error (it uses some precalculated quantities)
    %err_cp                  =     snorm_Y-2*sum(sum(Y_BA.*C_p))+...
%                                  sum(sum((C_p'*C_p).*temp_BA));
    err_cp                  =     sum((T(:)-norm_Y*kr(W{1}*C_p,V{1}*B_p,U{1}*A_p)*ones(R,1)).^2);
                                  
    % Evaluation of the relative error decrease
    diff_err_filt(1:end-1)  =     diff_err_filt(2:end);
    diff_err_filt(end)      =     abs((err_cp-err_cp_old)/err_cp(1));
    
    % Storing the last error
    err_cp_old       =     err_cp;
    err_cp_vec(t+1)  =     err_cp;
    
    % Objective value evaluation
    if mod(t*iter_err,100)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('Objective value : %g  \tit : %d\n',err_cp,t*iter_err)
    end
 
    
end
%--------------------------Algorithm iterates-----------------------------%    

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%

    
otherwise
disp('Unknown algorithm. Choose gradient, proximal or all-at-once.')

end

A   =     U{1}*A_p;
B   =     V{1}*B_p;
C   =     W{1}*C_p;
A(A<0)    =     0;
B(B<0)    =     0;
C(C<0)    =     0;

norm_A_u  =     sqrt(sum(A.^2));
norm_B_u  =     sqrt(sum(B.^2));

A   =     A.*repmat(1./norm_A_u,dim_u(1),1);
B   =     B.*repmat(1./norm_B_u,dim_u(2),1);
C   =     C.*repmat(norm_Y*norm_A_u.*norm_B_u,dim_u(3),1);

% Deleting zero error trailing at the end
err_cp_vec(err_cp_vec==0)=[];
%-------------------------------------------------------------------------%
end
