%-----------------------------prox_cp_test--------------------------------%
% This file tests the Prox-CP algorithm.
%
%                 For type='gradient', the data fitting term is considered
%                 as a smooth function and alternating gradient gradient
%                 is used.
%
%                 For type='proximal', the data fitting term is considered
%                 as a smooth function and alternating gradient gradient
%                 is used.
%
%                 For type='all-at-once', the data fitting term is conside-
%                 red as a smooth function and an all-at-once gradient is
%                 used.
%
% The test is done for nonnegative CP decomposition in the compressed
% domain, which is the same setting as in PROCO-ALS.
%
% External functions:
%
% - kr.m              : Khatri-Rao product;
% - proco_als.m       : Prox-ALS algorithm;
% - amb_correct.m     : Matching Permutation for two CP models.
%
% List of updates                 -     06/07/2015  -     J. E. Cohen
%                                       Creation of the file
%                                 -     07/10/2015  -     R. C. Farias
%                                       Modified file with prox-CP
%                                       function
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% Uncompressed dimensions
dim_u     =     [400 400 400];
% Compressed dimensions
dim_c     =     [5 5 5];
% Number of components
R         =     5;
% Standard deviations of the measurements noise (reduce for type='simplex')
sigma_n   =     1;
% Number of outer iterations (update of all factors)
iter_out  =     5000;
% Number of inner iteration  (required for type='gradient' and 'proximal')
iter_in   =     1;
% Parameter rho of the primal-dual proximal algorithm (between 0 and 2)
rho       =     1.95;
% Parameter tau of the primal-dual proximal algorithm
tau       =     0.25;
% Parameter sigma of the primal-dual proximal algorithm
sigma     =     0.25;
%-------------------------------------------------------------------------%

%------------------------------Tensor generation--------------------------%
% Unnormalized nonnegative factors
A   =     abs(randn(dim_u(1),R));
B   =     abs(randn(dim_u(2),R));
C   =     abs(randn(dim_u(3),R));
% Clean data block (useful to evaluate the signal to noise ratio)
X   =     reshape(A*transpose(kr(C,B)),dim_u);
% Noisy data block
Y   =     X+sigma_n*randn(dim_u);
% Signal to noise ratio
SNR =     10*log10(sum(sum(sum(X.^2)))/sum(sum(sum((Y-X).^2))));
% Delete clean data block 
clear     X;
%-------------------------------------------------------------------------%

%---------------------Operators and proximal mappings---------------------%
% Truncated HOSVD - Output compressed tensor and decompression operators
[G,U{1},V{1},W{1}]    =     hosvd(Y./sqrt(sum(sum(sum(Y.^2)))),dim_c,...
                                                                 'random');
% Proximal mappings (Projection on the nonnegative orthant)
% For a function g(x) this mapping is 
%                    sigma*prox_{g/sigma}(x/sigma)
prox_A{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity
prox_B{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity
prox_C{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity
%-------------------------------------------------------------------------%

%-----------------------------Initialization------------------------------%
% Factors in the uncompressed domain
A_0 =     abs(randn(dim_u(1),R));
B_0 =     abs(randn(dim_u(2),R));
C_0 =     abs(randn(dim_u(3),R));
% Normalization
A_0 =     A_0.*repmat(1./sqrt(sum(A_0.^2)),dim_u(1),1);
B_0 =     B_0.*repmat(1./sqrt(sum(B_0.^2)),dim_u(2),1);
C_0 =     C_0.*repmat(1./sqrt(sum(C_0.^2)),dim_u(3),1);
% Initialization of the primal factors - factors in the compressed domain
A_p_0     =     (U{1}')*A_0; 
B_p_0     =     (V{1}')*B_0;
C_p_0     =     (W{1}')*C_0;
% Initialization of the dual factors
A_d_0{1}  =     zeros(dim_u(1),R); 
B_d_0{1}  =     zeros(dim_u(2),R);
C_d_0{1}  =     zeros(dim_u(3),R);
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%

%----------------------------------Prox-ALS-------------------------------%
tic
%----------------------Alternating gradient version-----------------------%
type=     'gradient';
[A_grad,B_grad,C_grad,err_grad_vec] =   prox_cp(G,type,iter_out,iter_in...
                                             ,U,V,W,prox_A,prox_B,prox_C...
                                                          ,rho,tau,sigma...
                                                      ,A_p_0,B_p_0,C_p_0...
                                                       ,A_d_0,B_d_0,C_d_0);
% Uncompressed factors A and B
A_grad          =     U{1}*A_grad;
B_grad          =     V{1}*B_grad;
C_grad          =     W{1}*C_grad;
% Projection
A_grad(A_grad<0)     =     0;
B_grad(B_grad<0)     =     0;
C_grad(C_grad<0)     =     0;
% Normalization
norm_A_u        =     sum(A_grad);
norm_B_u        =     sum(B_grad);
A_grad    =     A_grad.*repmat(1./norm_A_u,dim_u(1),1);
B_grad    =     B_grad.*repmat(1./norm_B_u,dim_u(2),1);
C_grad          =     C_grad.*repmat(norm_A_u.*norm_B_u,dim_u(3),1);
% Correcting permutation and sign ambiguities
[A_grad,B_grad,C_grad]   =     amb_correct(A_grad,B_grad,C_grad,A,B,C);
%-------------------------------------------------------------------------%
toc
tic
%----------------------Alternating proximal version-----------------------%
type=     'proximal';
[A_prox,B_prox,C_prox,err_prox_vec] =   prox_cp(G,type,iter_out,iter_in...
                                             ,U,V,W,prox_A,prox_B,prox_C...
                                                          ,rho,tau,sigma...
                                                      ,A_p_0,B_p_0,C_p_0...
                                                       ,A_d_0,B_d_0,C_d_0);
% Uncompressed factors A and B
A_prox          =     U{1}*A_prox;
B_prox          =     V{1}*B_prox;
C_prox          =     W{1}*C_prox;
% Projection
A_prox(A_prox<0)     =     0;
B_prox(B_prox<0)     =     0;
C_prox(C_prox<0)     =     0;
% Normalization
norm_A_u        =     sum(A_prox);
norm_B_u        =     sum(B_prox);
A_prox    =     A_prox.*repmat(1./norm_A_u,dim_u(1),1);
B_prox    =     B_prox.*repmat(1./norm_B_u,dim_u(2),1);
C_prox          =     C_prox.*repmat(norm_A_u.*norm_B_u,dim_u(3),1);
% Correcting permutation and sign ambiguities
[A_prox,B_prox,C_prox]   =     amb_correct(A_prox,B_prox,C_prox,A,B,C);
%-------------------------------------------------------------------------%
toc
tic
%----------------------All-at-once gradient version-----------------------%
type=     'all-at-once';
[A_all,B_all,C_all,err_all_vec]     =   prox_cp(G,type,iter_out,iter_in...
                                             ,U,V,W,prox_A,prox_B,prox_C...
                                                          ,rho,tau,sigma...
                                                      ,A_p_0,B_p_0,C_p_0...
                                                       ,A_d_0,B_d_0,C_d_0);
% Uncompressed factors A and B
A_all           =     U{1}*A_all;
B_all           =     V{1}*B_all;
C_all           =     W{1}*C_all;
% Projection
A_all(A_all<0)  =     0;
B_all(B_all<0)  =     0;
C_all(C_all<0)  =     0;
% Normalization
norm_A_u        =     sum(A_all);
norm_B_u        =     sum(B_all);
A_all     =     A_all.*repmat(1./norm_A_u,dim_u(1),1);
B_all     =     B_all.*repmat(1./norm_B_u,dim_u(2),1);
C_all           =     C_all.*repmat(norm_A_u.*norm_B_u,dim_u(3),1);
% Correcting permutation and sign ambiguities
[A_all,B_all,C_all]   =     amb_correct(A_all,B_all,C_all,A,B,C);
%-------------------------------------------------------------------------%
toc
%-------------------------------------------------------------------------%
