function [A,B,C,err_als,temps]     =     qp_als(G,iter,inner,dim_c,dim_u,A_0,B_0,C_0,U,V,W,T)
%-------------------------------------------------------------------------%
% [A_c,B_c,C_c,err_als_vec]=     proco_als(Y,iter,dim_c,A_0,B_0,C_0)
%
% Compressed and projected ALS algorithm for large-scale 3-way nonnegative
% tensor decomposition.
%
% The inputs are:
% 
% - Y           : data block.
% - iter        : maximum number of iterates.
% - dim_c       : vector with the dimensions of the compressed model.
% - A_0,B_0,C_0 : initial parameters for the factors.
%
% The outputs are:
% 
% - A           : first factor.
% - B           : second factor.
% - C           : third factor.
% - Lambda      : amplitude of the R components
% - T_als       : reconstructed tensor
% - err_als     : final error
%
% List of updates                 -     28/08/2014  -     Rodrigo Cabral
%                                       Creation of the file
%                                 -     02/10/2014  -     Jeremy Cohen
%                                       Reshaping, modified outputs
%                                 -     07/01/2015  -     Jeremy Cohen
%                                       Added Lambda as output
%                                 -     08/06/2015  -     Jeremy Cohen
%                                       QuadProg version
%                                 -     06/07/2015  -     Jeremy Cohen
%                                       Added Time output.
%-------------------------------------------------------------------------%


%--------------------------Tensor compression-----------------------------%
R    =    size(A_0,2);
% Index for n-mode matricization of compressed data
J_c  =    prod(dim_c)./dim_c;
% Truncated core tensors (compressed data)
norm_G          =     sqrt(G(:)'*G(:));
% Normalized G
G               =     G./norm_G;

% Three different unfoldings of the tensor
G_mode    =     cell(3);
% Data tensor unfolding
G_mode{1} =     reshape(permute(G,[1,2,3]),dim_c(1),J_c(1));  
G_mode{2} =     reshape(permute(G,[2,1,3]),dim_c(2),J_c(2));
G_mode{3} =     reshape(permute(G,[3,1,2]),dim_c(3),J_c(3));



%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%

% Storing initial condition on the compressed space
A_c  =    (U')*A_0;
B_c  =    (V')*B_0;
C_c  =    (W')*C_0;

%-------------------------------------------------------------------------%

fprintf('\n\n')
fprintf('\n Active set QuadProg  - ASQP \n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
temps(1)  =     1;
%err_als(1)=     sum(sum((G_mode{1}-A_c*kr(C_c,B_c)').^2));
err_als(1) =     sum((T(:)-kr(C_0,B_0,A_0)*ones(R,1)).^2);

while     t<iter
    
time = tic;    
    
    % Iteration counter
    t=t+1;
    
    % QP solution for each factor
    
    opts = optimoptions('quadprog','Algorithm','active-set','Display','off','MaxIter',inner);
    
    % A factor update
    [A_c]     =     quadprog(kron(eye(dim_c(1)),(C_c'*C_c).*(B_c'*B_c)),-vec(G_mode{1})'*kron(eye(dim_c(1)),kr(C_c,B_c)),-kron(U,eye(R)),zeros(R*dim_u(1),1),[],[],[],[],vec(A_c),opts);
    A_c       =     reshape(A_c,R,dim_c(1))';
    A_c       =     A_c.*repmat(1./sqrt(sum(A_c.^2)),dim_c(1),1);
        
    % B factor update
    [B_c]     =     quadprog(kron(eye(dim_c(2)),(C_c'*C_c).*(A_c'*A_c)),-vec(G_mode{2})'*kron(eye(dim_c(2)),kr(C_c,A_c)),-kron(V,eye(R)),zeros(R*dim_u(2),1),[],[],[],[],vec(B_c),opts);
    B_c       =     reshape(B_c,R,dim_c(2))';
    B_c       =     B_c.*repmat(1./sqrt(sum(B_c.^2)),dim_c(2),1);
        
    % C factor update
    [C_c]     =     quadprog(kron(eye(dim_c(3)),(B_c'*B_c).*(A_c'*A_c)),-vec(G_mode{3})'*kron(eye(dim_c(3)),kr(B_c,A_c)),-kron(W,eye(R)),zeros(R*dim_u(3),1),[],[],[],[],vec(C_c),opts);
    C_c       =     reshape(C_c,R,dim_c(3))';
    
    if (mod(t,50) == 0 )%&& t<iter)
    fprintf('\tit : %d\n',t)    
    end
temps(t+1)=toc(time); 
    
%err_als(t+1) =     sum(sum((G_mode{1}-A_c*kr(C_c,B_c)').^2));    
err_als(t+1) =     sum((T(:)-norm_G*kr(W*C_c,V*B_c,U*A_c)*ones(R,1)).^2);   
   

end
    % Error computation
A   =     U*A_c;
B   =     V*B_c;
C   =     W*C_c;
A(A<0)    =     0;
B(B<0)    =     0;
C(C<0)    =     0;

norm_A_u  =     sqrt(sum(A.^2));
norm_B_u  =     sqrt(sum(B.^2));

A   =     A.*repmat(1./norm_A_u,dim_u(1),1);
B   =     B.*repmat(1./norm_B_u,dim_u(2),1);
C   =     C.*repmat(norm_G*norm_A_u.*norm_B_u,dim_u(3),1);


fprintf('Reconstruction error : %g  ',err_als(end)) 

A(isnan(A))     =     0;
B(isnan(B))     =     0;
C(isnan(C))     =     0;
%-------------------------------------------------------------------------%    